package net.wanji.opt.controller.induce;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.framework.rest.ValidationGroups;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.dao.mapper.induce.GreenwaveInducesHistMapper;
import net.wanji.opt.dto.induce.GreenwaveInducesDTO;
import net.wanji.opt.dto.induce.InduceDTO;
import net.wanji.opt.entity.DeviceInduces;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.GreenwaveInducesHist;
import net.wanji.opt.entity.InduceTemplate;
import net.wanji.opt.service.DeviceInducesService;
import net.wanji.opt.service.induce.GreenwaveInducesHistService;
import net.wanji.opt.service.induce.GreenwaveInducesService;
import net.wanji.opt.service.induce.InduceTemplateService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static org.codehaus.groovy.runtime.DefaultGroovyMethods.collect;

/**
 * <p>
 * 绿波诱导下发状态信息
 * </p>
 *
 * @author guoliang.dong
 * @since 2024-11-22
 */
@Api(value = "GreenwaveInducesHistController", description = "绿波-绿波诱导下发状态信息")
@RestController
@RequestMapping("/greenwaveInducesHist")
@Slf4j
public class GreenwaveInducesHistController {

    @Resource
    private GreenwaveInducesHistService greenwaveInducesHistService;

    @Resource
    private GreenwaveInducesHistMapper greenwaveInducesHistMapper;

    @Resource
    private GreenwaveInducesService greenwaveInducesService;

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;

//    @Resource
//    private InduceTemplateService induceTemplateService;

    @Resource
    private DeviceInducesService deviceInducesService;

    /**
     * 获取所有记录
     *
     * @return JsonViewObject
     */
    @ApiOperation(value = "获取所有记录", notes = "获取所有记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byAll", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getAll() {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start = System.currentTimeMillis();
        try {
            List<GreenwaveInducesDTO> list = new ArrayList<>();
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list();
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            this.greenwaveInducesHistService.list().stream().forEach(o -> {
                GreenwaveInducesDTO greenwaveInducesDTO = new GreenwaveInducesDTO();
                greenwaveInducesDTO.setGreenId(o.getGreenId());
                greenwaveInducesDTO.setDir(o.getDir());
                //这里从绿波表里获取基本信息
                List<GreenwaveInfoPO> greenwaveInfoByGreenId = greenwaveInfoPOList.stream().filter(n -> n.getId().equals(o.getGreenId())).collect(Collectors.toList());// 过滤条件// 收集结果
                if (Objects.nonNull(greenwaveInfoByGreenId) && greenwaveInfoByGreenId.size() > 0) {
                    greenwaveInducesDTO.setGreenName(greenwaveInfoByGreenId.get(0).getName());
                    greenwaveInducesDTO.setDesignSpeed(greenwaveInfoByGreenId.get(0).getDesignSpeed());
                }
                greenwaveInducesDTO.setStatus(o.getStatus());
                greenwaveInducesDTO.setStrategyId(o.getStrategyId());
                greenwaveInducesDTO.setStrategyName(o.getStrategyName());
                greenwaveInducesDTO.setMinSpeed(o.getMinSpeed());
                greenwaveInducesDTO.setMinSpeed(o.getMaxSpeed());
                greenwaveInducesDTO.setControlOptTimes(o.getControlOptTimes());
                //获取诱导屏列表
                List<InduceDTO> induceDTOList = new ArrayList<>();
                greenwaveInducesList.stream().filter(x -> Objects.nonNull(x.getGreenId()) && x.getGreenId().equals(o.getGreenId())).forEach(n -> {
                    InduceDTO induceDTO = new InduceDTO();
                    induceDTO.setId(n.getId());
                    induceDTO.setEquipCode(n.getEquipCode());
                    induceDTO.setStatus(o.getStatus());
                    //获取屏幕的设备信息
                    List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(m -> Objects.nonNull(m.getEquipCode()) && m.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                    if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                        induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                        induceDTO.setRidDir(n.getDir());//
                        induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());//
                        induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());//
                        induceDTO.setWkt(deviceInduces.get(0).getWkt());
                    }
                    //关联模板信息
                    //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(x->Objects.isNull(x.getEquipCode()) || x.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList()));
                    induceDTOList.add(induceDTO);
                });
                greenwaveInducesDTO.setInduceList(induceDTOList);
                greenwaveInducesDTO.setEquipCount(induceDTOList.size());
                list.add(greenwaveInducesDTO);
            });
            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonView;
    }


    /**
     * 根据条件查询记录
     *
     * @param GreenwaveInducesHist 查询条件
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据条件查询记录", notes = "根据条件查询记录", response = JsonViewObject.class, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/byCondition", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject getByWhere(@ApiParam(value = "查询条件", required = true) @RequestBody @Validated({ValidationGroups.Query.class}) GreenwaveInducesHist GreenwaveInducesHist){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String jsonStr = JSON.toJSONString(GreenwaveInducesHist);
        try {
            //参数校验过程中修改，兼容实现，restful入参不用map，便于进行参数逐个校验
            Map params = JSONObject.parseObject(JSONObject.toJSONString(GreenwaveInducesHist), Map.class);
//            Map<String, Object> params = new HashMap<>();
//            JSONObject.parseObject(JSONObject.toJSONString(GreenwaveInducesHist), Map.class).forEach((k,v) -> params.put(StringUtils.camelToCapital(k.toString()).toLowerCase(), v));
            List<GreenwaveInducesDTO> list = new ArrayList<>();
            List<GreenwaveInducesHist> greenwaveInducesHistList = this.greenwaveInducesHistMapper.findByMap(params);
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            List<GreenwaveInduces> greenwaveInducesList = greenwaveInducesService.list();
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            greenwaveInducesHistList.stream().forEach(o -> {
                GreenwaveInducesDTO greenwaveInducesDTO = new GreenwaveInducesDTO();
                greenwaveInducesDTO.setGreenId(o.getGreenId());
                greenwaveInducesDTO.setDir(o.getDir());
                //这里从绿波表里获取基本信息
                List<GreenwaveInfoPO> greenwaveInfoByGreenId = greenwaveInfoPOList.stream().filter(n -> n.getId().equals(o.getGreenId())).collect(Collectors.toList());// 过滤条件// 收集结果
                if (Objects.nonNull(greenwaveInfoByGreenId) && greenwaveInfoByGreenId.size() > 0) {
                    greenwaveInducesDTO.setGreenName(greenwaveInfoByGreenId.get(0).getName());
                    greenwaveInducesDTO.setDesignSpeed(greenwaveInfoByGreenId.get(0).getDesignSpeed());
                    greenwaveInducesDTO.setWkt(greenwaveInfoByGreenId.get(0).getWkt());
                }
                greenwaveInducesDTO.setStatus(o.getStatus());
                greenwaveInducesDTO.setStrategyId(o.getStrategyId());
                greenwaveInducesDTO.setStrategyName(o.getStrategyName());
                greenwaveInducesDTO.setMinSpeed(o.getMinSpeed());
                greenwaveInducesDTO.setMinSpeed(o.getMaxSpeed());
                greenwaveInducesDTO.setControlOptTimes(o.getControlOptTimes());
                //获取诱导屏列表
                List<InduceDTO> induceDTOList = new ArrayList<>();
                greenwaveInducesList.stream().filter(x -> Objects.nonNull(x.getGreenId()) && x.getGreenId().equals(o.getGreenId())).forEach(n -> {
                    InduceDTO induceDTO = new InduceDTO();
                    induceDTO.setId(n.getId());
                    induceDTO.setEquipCode(n.getEquipCode());
                    induceDTO.setStatus(o.getStatus());
                    //获取屏幕的设备信息
                    List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(m -> Objects.nonNull(m.getEquipCode()) && m.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                    if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                        induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                        induceDTO.setRidDir(n.getDir());//
                        induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());//
                        induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());//
                        induceDTO.setWkt(deviceInduces.get(0).getWkt());
                    }
                    //关联模板信息
                    //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(x -> Objects.isNull(x.getEquipCode()) || x.getEquipCode().equals(n.getEquipCode())).collect(Collectors.toList()));
                    induceDTOList.add(induceDTO);
                });
                greenwaveInducesDTO.setInduceList(induceDTOList);
                greenwaveInducesDTO.setEquipCount(induceDTOList.size());
                list.add(greenwaveInducesDTO);
            });
            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getByWhere error，jsonStr:{}", this.getClass().getSimpleName(), jsonStr, e);
        }
        return jsonView;
    }

    /**
     * 根据id查询记录
     *
     * @param id
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据id查询记录", notes = "根据id查询记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getById(@ApiParam(value = "记录的id", required = true, example = "1") @PathVariable("id") @NotBlank(message = "查询id不能为空") String id){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            GreenwaveInducesHist GreenwaveInducesHist = this.greenwaveInducesHistService.getById(id);
            jsonView.success(GreenwaveInducesHist);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("AbstractRestServerImpl getById error, id:{}", id, e);
        }
        return jsonView;
    }

    /**
     * 根据id删除
     *
     * @param ids
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据多个id删除记录", notes = "根据多个id删除记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "ids", dataType = "String", required = true, value = "多个记录id，用逗号分隔", example = "1,2")
    })
    @GetMapping(value = "/deleting", produces = MediaType.APPLICATION_JSON)
    JsonViewObject deleteByIds(@QueryParam("ids") @NotBlank(message = "删除ids不能为空") String ids){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String[] idArray = ids.split(",");
        try {
            if (idArray.length > 0) {
                for (String id : idArray) {
                    jsonView = this.greenwaveInducesHistService.removeByIds(Arrays.asList(ids.split(",")))?jsonView.success():jsonView.fail();;
                }
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("DELETE_FAILED_MSG"));
            log.error("AbstractRestServerImpl deleteByIds error, id:{}", ids, e);
        }
        return jsonView;
    }

    /**
     * 新建记录
     *
     * @param GreenwaveInducesHist
     * @return JsonViewObject
     */
    @ApiOperation(value = "新建记录", notes = "新建记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/creating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject save(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Create.class}) GreenwaveInducesHist GreenwaveInducesHist){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            if (GreenwaveInducesHist != null) {
                jsonView = this.greenwaveInducesHistService.saveOrUpdate(GreenwaveInducesHist)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("SAVE_FAILED_MSG"));
            log.error("AbstractRestServerImpl save error, jsonStr:{}", JSON.toJSONString(GreenwaveInducesHist), e);
        }
        return jsonView;
    }

    /**
     * 修改记录
     *
     * @param GreenwaveInducesHist
     * @return
     */
    @ApiOperation(value = "修改记录", notes = "修改记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject update(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Update.class}) GreenwaveInducesHist GreenwaveInducesHist){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            if (GreenwaveInducesHist != null) {
                jsonView = this.greenwaveInducesHistService.saveOrUpdate(GreenwaveInducesHist)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("UPDATE_FAILED_MSG"));
            log.error("AbstractRestServerImpl update error, jsonStr:{}", JSON.toJSONString(GreenwaveInducesHist), e);
        }
        return jsonView;
    }
}
